import processing.core.*; 
import processing.xml.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class MIT_Light extends PApplet {

PImage online;
int majorityColor;
int minorityColor;
String url;



public void setup() {
  size (300,300);
  background(70);
  url = "http://mit.edu/img/MIT_logo.gif";
  online = loadImage(url, "gif");
  imageColors();
  noStroke();
}



public void draw() {
  image(online, 45, 10, online.width*(2.7f), online.height*(2.7f)); // scale image
  
  if(majorColorHover()){      // check to see if the mouse is hovering over the major color adjustment bar
    displayMajorAdjust();     // display the RGB adjustment bars for the major color
  }
  else if(minorColorHover()){ // check to see if the mouse is hovering over the minor color adjustment bar
    displayMinorAdjust();     // display the RGB adjustment bars for the minor color
  }
  else{
    imageColors();            // if the mouse is not hovering over either bar then just display the solid color bars
  }
    
}



public void imageColors(){
  online.loadPixels(); // load the image pixels into the array
    
  majorityColor = online.pixels[0];             // the majority color can be found at index 0
  int majorityR = (majorityColor >> 16) & 0xFF; // get the red component of the majority color
  int majorityG = (majorityColor >> 8) & 0xFF;  // get the green component of the majority color
  int majorityB = majorityColor & 0xFF;         // get the blue component of the majority color
  minorityColor = online.pixels[1140];          // the minority color can be found at index 1140
  int minorityR = (minorityColor >> 16) & 0xFF; // get the red component of the minority color
  int minorityG = (minorityColor >> 8) & 0xFF;  // get the green component of the minority color
  int minorityB = minorityColor & 0xFF;         // get the blue component of the minority color
  
  if(majorColorHover() == false && minorColorHover() == false){ // if the mouse is not over either adjustment bar then display the solid color bar
    fill(majorityR,majorityG,majorityB);        // change the fill color to the RGB values of the major color
    rect(0,130,300,75);                         // draw the major color solid adjustment bar
    fill(minorityR,minorityG,minorityB);        // change the fill color to the RGB values of the minor color
    rect(0,215,300,75);                         // draw the minor color solid adjustment bar
  }
}



public boolean majorColorHover(){       // checks if the mouse is hovering ove the major adjustment bar
  if(mouseY>130 && mouseY<205){  // return true if it is
    return true;
  }
  return false;                  // else return false
}



public boolean minorColorHover(){      // checks if the mouse is hovering ove the minor adjustment bar
  if(mouseY>215 && mouseY<290){ // return true if it is
    return true;
  }
  return false;                 // else return false
}



public void displayMajorAdjust(){
  fill(40);
  rect(0,130,300,25); // overlay major red slider background 
  rect(0,155,300,25); // overlay major green slider background
  rect(0,180,300,25); // overlay major blue slider background
  
  int redMap = (int)map((majorityColor >> 16) & 0xFF,0,255,0,300);  // map the red value so it can be displayed as a rectangle slider
  int greenMap = (int)map((majorityColor >> 8) & 0xFF,0,255,0,300); // map the green value so it can be displayed as a rectangle slider
  int blueMap = (int)map(majorityColor & 0xFF,0,255,0,300);         // map the blue value so it can be displayed as a rectangle slider
   
  fill(255,0,0);           // change the fill color to red
  rect(0,130,redMap,25);   // overlay major red slider
  fill(0,255,0);           // change the fill color to green
  rect(0,155,greenMap,25); // overlay major green slider
  fill(0,0,255);           // change the fill color to blue
  rect(0,180,blueMap,25);  // overlay major blue slider
}



public void displayMinorAdjust(){
  fill(40);
  rect(0,215,300,25); // overlay minor red slider background
  rect(0,240,300,25); // overlay minor green slider background
  rect(0,265,300,25); // overlay minor blue slider background
  
  int redMap = (int)map((minorityColor >> 16) & 0xFF,0,255,0,300);  // map the red value so it can be displayed as a rectangle slider
  int greenMap = (int)map((minorityColor >> 8) & 0xFF,0,255,0,300); // map the green value so it can be displayed as a rectangle slider
  int blueMap = (int)map(minorityColor & 0xFF,0,255,0,300);         // map the blue value so it can be displayed as a rectangle slider
  
  fill(255,0,0);           // change the fill color to red
  rect(0,215,redMap,25);   // overlay minor red slider
  fill(0,255,0);           // change the fill color to green
  rect(0,240,greenMap,25); // overlay minor green slider
  fill(0,0,255);           // change the fill color to blue
  rect(0,265,blueMap,25);  // overlay minor blue slider
}



public void mousePressed(){                // runs if the mouse was pressed
  if(mouseButton == LEFT){          // checks to see if the left button was pressed   
    adjustingColors();              // if the left button was clicked then run adjustingColors to adjust the colors
  }
  else if(mouseButton == RIGHT){    // checks to see if the right button was pressed
    //online = loadImage(url, "gif"); // if the right button was clicked then restore the image to the one currently online
    //imageColors();                  // run imageColors to update the adjustment bars back to normal
  }
}



public void mouseDragged(){  // runs if the mouse was clicked and then dragged
   adjustingColors(); // run adjustingColors to adjust the colors
}



public void adjustingColors(){ // adjusts the RGB colors of the major and minor colors according to the sliders
  if(mouseY > 130 && mouseY < 205){ // Major color
    int majorNewColor;
    int updatedRed = (majorityColor >> 16) & 0xFF;
    int updatedGreen = (majorityColor >> 8) & 0xFF;
    int updatedBlue = majorityColor & 0xFF;
    int majorRow = (mouseY-130) / 25;
      
    if(majorRow == 0){
      updatedRed = (int)map(mouseX,0,300,0,255);
      fill(255,0,0);
      rect(0,130,mouseX,25); // overlay major red slider
    }
    if(majorRow == 1){
      updatedGreen = (int)map(mouseX,0,300,0,255);
      fill(0,255,0);
      rect(0,155,mouseX,25); // overlay major green slider
    }
    if(majorRow == 2){
      updatedBlue = (int)map(mouseX,0,300,0,255);
      fill(0,0,255);
      rect(0,180,mouseX,25); // overlay major blue slider
    }
    updateColors(0,updatedRed,updatedGreen,updatedBlue);
  }
    
  if(mouseY > 215 && mouseY < 290){  // Minor color
    int minorNewColor;
    int updatedRed = (minorityColor >> 16) & 0xFF;
    int updatedGreen = (minorityColor >> 8) & 0xFF;
    int updatedBlue = minorityColor & 0xFF;
    int minorRow = (mouseY-215) / 25;
      
    if(minorRow == 0){
      updatedRed = (int)map(mouseX,0,300,0,255);
      fill(255,0,0);
      rect(0,215,mouseX,25); // overlay minor red slider
    }
    if(minorRow == 1){
      updatedGreen = (int)map(mouseX,0,300,0,255);
      fill(0,255,0);
      rect(0,240,mouseX,25); // overlay minor green slider
    }
    if(minorRow == 2){
      updatedBlue = (int)map(mouseX,0,300,0,255);
      fill(0,0,255);
      rect(0,265,mouseX,25); // overlay minor blue slider
    }     
    updateColors(1140,updatedRed,updatedGreen,updatedBlue);
  }
}




public void updateColors(int findIndex, int newRed, int newGreen, int newBlue){ // changes all colors that match the color of the given index to the RGB parameters
  online.loadPixels();
  int currColor = online.pixels[findIndex] ;
  int currR = (currColor >> 16) & 0xFF;;
  int currG = (currColor >> 8) & 0xFF;;
  int currB = currColor & 0xFF;;
  
  for(int x=0; x<online.width*online.height; x++){
     if(online.pixels[x] == currColor){
       online.pixels[x] =  color(newRed,newGreen,newBlue);
     }
  }
  
  online.updatePixels();
  imageColors();
}




  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#FFFFFF", "MIT_Light" });
  }
}
